

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity debounce is
    Port ( clk : in std_logic;  
           clk_div: in std_logic;
           reset_n : in std_logic;
           button: in std_logic;
           int_button: out std_logic
           );            
end debounce;



architecture behavioral of debounce is

type debounce_state_type is (start,timer,clear);
signal debounce_state: debounce_state_type;

signal button_ff: std_logic; -- Metastability flop 1
signal button_ff1: std_logic;-- Metastability flop 2
signal count_start: std_logic;
signal count: std_logic_vector(5 downto 0);
signal count_term: std_logic;

begin

-- Metastability circuit
process (clk,reset_n)
begin
	if reset_n = '0' then		
       button_ff  <= '0';
       button_ff1 <= '0';
     elsif clk'event and clk = '1' then
	   button_ff  <= button;
	   button_ff1 <= button_ff;
	end if;
end process;

-- debounce counter
process (clk_div,reset_n)
begin
	if reset_n = '0' then		
       count  <= "000000";
       count_term <= '0';
     elsif clk_div'event and clk_div = '1' then
	   if count = "100110" and count_start = '1' then
	      count_term <= '1';
	      count <= "000000";
	    elsif count_start = '1' then
	       count <= count + 1;
	       count_term <= '0';
	      else
	        count <= "000000";
	        count_term <= '0';
	    end if;
	end if;
end process;



debounce_machine: process (clk,reset_n)
begin
	if reset_n = '0' then		
		debounce_state <= start;
        count_start <= '0';
        int_button <= '0';
    elsif clk'event and clk = '1' then
	   case debounce_state is
		when start =>
		     if button_ff1 = '1' then
		        count_start <= '1';
		        int_button <= '0';
		        debounce_state <= timer;  
		      else
		        count_start <= '0';
		        int_button <= '0';
		        debounce_state <= start; 
		      end if;                 
		when timer =>
		      if count_term = '1' and button_ff1 = '1' then
		          count_start <= '0';
		          int_button <= '1';    -- set interrupt 
		          debounce_state <= clear;  
		        elsif count_term = '1' and button_ff1 = '0' then
		            count_start <= '0';
		            int_button <= '0';
		            debounce_state <= clear;  
		          else
		             count_start <= '1';
		             int_button <= '0';
		             debounce_state <= timer;  
		       end if;  
        when clear =>
                if count_term = '1' then 
   		           count_start <= '0';
		           int_button <= '0';
		           debounce_state <= clear;
		         elsif button_ff1 = '0' then
		           count_start <= '0';
		           int_button <= '0';
		           debounce_state <= clear;
		          else
		            count_start <= '0';
		            int_button <= '0';
		            debounce_state <= start;  
                 end if;      
        when others =>
            debounce_state <= start;
            count_start <= '0';
		    int_button <= '0';
      end case;
   end if;
end process;



end behavioral;
